#include "ShaderConstants.fxh"

struct PS_Input
{
		float4 position : SV_Position;
		float4 fog : FOG;
		float2 pos : POS;
};

struct PS_Output
{
		float4 color : SV_Target;
};

float3 mod289(float3 x) { return x - floor(x * (1.0 / 289.0)) * 289.0; }
float2 mod289(float2 x) { return x - floor(x * (1.0 / 289.0)) * 289.0; }
float3 permute(float3 x) { return mod289(x*(x * 34.0 + 1.0)); }

float snoise(float2 v) {
	float4 C = float4(
		0.211324865405187,   // (3.0-sqrt(3.0))/6.0
		0.366025403784439,   // 0.5*(sqrt(3.0)-1.0)
		-0.577350269189626,  // -1.0 + 2.0 * C.x
		0.024390243902439);  // 1.0 / 41.0

	float2 i  = floor(v + dot(v, C.yy));
	float2 x0 = v -   i + dot(i, C.xx);
	float2 i1  = x0.x > x0.y ? float2(1.0, 0.0) : float2(0.0, 1.0);
	float4 x12 = x0.xyxy + C.xxzz;
	x12.xy -= i1;

	i = mod289(i);
	float3 p =permute(permute(i.y + float3(0.0, i1.y, 1.0)) + i.x + float3(0.0, i1.x, 1.0));

	float3 m = max(0.5 - float3(dot(x0, x0), dot(x12.xy, x12.xy), dot(x12.zw, x12.zw)), 0.0);
	m = m*m;
	m = m*m;

	float3 x  = 2.0 * frac(p * C.www) - 1.0;
	float3 h  = abs(x) - 0.5;
	float3 ox = round(x);
	float3 a0 = x - ox;

	m *= rsqrt(a0 * a0 + h * h);
	float3 g;
	g.x  = a0.x  * x0.x   + h.x  * x0.y;
	g.yz = a0.yz * x12.xz + h.yz * x12.yw;
	return 130.0 * dot(m, g);
}

float fBM(int octaves, float lowerBound, float upperBound, float2 st) {
	float value = 0.0;
	float amplitude = 0.5;
	for (int i = 0; i < octaves; i++) {
		value += amplitude * (snoise(st) * 0.5 + 0.5);
		if (value >= upperBound) {break;}
		else if (value + amplitude <= lowerBound) {break;}
		st        *= 2.0;
		st.x      -=TIME/256.0*float(i+1);
		amplitude *= 0.5;
	}
	return smoothstep(lowerBound, upperBound, value);
}


ROOT_SIGNATURE
void main(in PS_Input PSInput, out PS_Output PSOutput)
{
	//DATABASE
	float day = smoothstep(0.15,0.25,FOG_COLOR.g);
	float weather = smoothstep(0.8,1.0,FOG_CONTROL.y);
	float ss = smoothstep(0.0,0.5,FOG_COLOR.r-FOG_COLOR.g);

	float4 col = lerp(CURRENT_COLOR,FOG_COLOR*1.1,smoothstep(.1,.4,PSInput.fog))+/*夜空の色*/float4(0.0,0.02,0.1,0.0);

		//AURORA
		float aflag = (1.-day)*weather;
		if(aflag > 0.){
			float2 apos = float2(PSInput.pos.x-TIME/256.,PSInput.pos.y*10.);
			apos.y += sin(PSInput.pos.x*20.+TIME*.1)*.15;
			float3 ac = lerp(/*オーロラ色1*/float3(0.,.8,.4),/*オーロラ色2*/float3(.4,.2,.8),sin(apos.x+apos.y+TIME*.01)*.5+.5);
			float am = fBM(4,.5,1.,apos);
			col.rgb += ac*am*smoothstep(.5,0.,length(PSInput.pos))*aflag;
		}

		//CLOUDS
		float3 cc = lerp(.2,1.,day)*float3(.95+ss,1.,1.-ss);
		float lb = lerp(.1,.5,weather);
		float cm = fBM(6,lb,1.,PSInput.pos*3.-TIME*.001);
		col.rgb = lerp(col.rgb,cc,cm);

	PSOutput.color =lerp(col,FOG_COLOR,PSInput.fog);
}
